/*
 * Utils.java
 *
 * 作成日: 2009/10/03
 * Copyright(C)2008 Kyushu Institute of Technology., All right reserved. 
 */
package jp.ac.kyutech.SRP.Scripting;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import jp.go.aist.rtm.RTC.util.NVUtil;
import jp.go.aist.rtm.RTC.util.ORBUtil;

import org.apache.commons.lang.StringUtils;
import org.omg.CORBA.Any;

import _SDOPackage.NameValue;

/**
 * SDLEngine Scriptのユーティリティクラスです。
 * @author 
 */
public class Utils {

    /**
     * NameValue の配列から、指定された名前の NameValue を検索します。
     * @param nameValues
     * @param name
     * @return
     */
    public static NameValue find(NameValue[] nameValues, String name) {
        if (name == null)
            return null;
        for (int i = 0; i < (nameValues != null ? nameValues.length : 0); i++) {
            NameValue nameValue = nameValues[i];
            if (name.equals(nameValue.name)) {
                return nameValue;
            }
        }
        return null;
    }
    
    /**
     * @param nameValues
     * @param name
     * @return
     */
    public static String findValueAsString(NameValue[] nameValues, String name) {
        if (name == null)
            return null;
        for (int i = 0; i < (nameValues != null ? nameValues.length : 0); i++) {
            NameValue nameValue = nameValues[i];
            if (name.equals(nameValue.name)) {
                try {
                    return nameValue.value.extract_string();
                } catch (Exception e1) {
                    try {
                        return nameValue.value.extract_wstring();
                    } catch (Exception e2) {
                    }
                }
            }
        }
        return null;
    }
    
    @SuppressWarnings("unchecked")
    public static Map toMap(NameValue[] nameValues) {
        Map result = new java.util.HashMap();
        for (int i = 0; i < (nameValues != null ? nameValues.length : 0); i++) {
            try {
                NameValue nameValue = nameValues[i];
                if (nameValue != null && nameValue.name != null) {
                    result.put(nameValue.name, nameValue.value);
                }
            } catch (Exception e) {
            }
        }
        return result;
    }
    
    /**
     * 引数で与えたキー、値の配列からマップを構築する。
     * リテラル風にマップを定義するのに用いる。
     * 
     * toMap("a", 1, "b", 2);  だと {"a" => 1, "b" => 2} といったハッシュマップを作る。
     * 
     * @param keyAndValues
     * @return
     */
    @SuppressWarnings("unchecked")
    public static Map toMap(Object... keyAndValues) {
        Map result = new java.util.HashMap();
        for(int i = 0; i < keyAndValues.length; i+= 2) {
            Object key = keyAndValues[i];
            Object value = keyAndValues[i + 1];
            result.put(key, value);
        }
        return result;
    }

    /**
     * @param propertyMap
     * @return
     */
    @SuppressWarnings("unchecked")
    public static NameValue[] toNVList(Map propertyMap) {
        List nameValueList = new java.util.ArrayList();
        for(Iterator it =  propertyMap.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry entry = (Map.Entry )it.next();
            if (entry.getValue() instanceof Any) {
                nameValueList.add(NVUtil.newNVAny(entry.getKey().toString(), (Any )entry.getValue()));
            }
            else {
                nameValueList.add(newNVString(entry.getKey().toString(), entry.getValue().toString()));
            }
        }
        return (NameValue[] )nameValueList.toArray(new NameValue[nameValueList.size()]);
    }

    /**
     * @param keyAndValues
     * @return
     */
    @SuppressWarnings("unchecked")
    public static NameValue[] toNVList(Object... keyAndValues) {
        List nameValueList = new java.util.ArrayList();
        for(int i = 0; i < keyAndValues.length; i+= 2) {
            Object key = keyAndValues[i];
            Object value = keyAndValues[i + 1];
            if (value instanceof Any) {
                nameValueList.add(NVUtil.newNVAny(key.toString(), (Any )value));
            }
            else {
                nameValueList.add(newNVString(key.toString(), value.toString()));
            }
        }
        return (NameValue[] )nameValueList.toArray(new NameValue[nameValueList.size()]);
    }

    /**
     * @param name
     * @param value
     * @return
     */
    public static NameValue newNVString(final String name, String value) {
        NameValue nv = new NameValue();
        nv.name = name;
        Any any = ORBUtil.getOrb().create_any();
        if (StringUtils.isAsciiPrintable(value)) {
            any.insert_string(value);
        }
        else {
            any.insert_wstring(value);
        }
        nv.value = any;
        return nv;
    }
    
    /**
     * @param o1
     * @param o2
     * @return
     */
    public static boolean equals(Object o1, Object o2) {
        String s1 = null, s2 = null;
        if (o1 instanceof Any) {
            Any a1 = (Any )o1;
            try {
                s1 = a1.extract_string();
            } catch (Exception e1) {
                try {
                    s1 = a1.extract_wstring();
                } catch (Exception e2) {
                }
            }
        }
        else {
            s1 = (o1 != null ? o1.toString() : null);
        }
        if (o2 instanceof Any) {
            Any a2 = (Any )o2;
            try {
                s2 = a2.extract_string();
            } catch (Exception e1) {
                try {
                    s2 = a2.extract_wstring();
                } catch (Exception e2) {
                }
            }
        }
        else {
            s2 = (o2 != null ? o2.toString() : null);
        }
        return StringUtils.equals(s1, s2);
    }
    
}
